@@ -1,21 +1,41 @@
 from __future__ import annotations
 
-from typing import Iterator, NamedTuple
+from collections.abc import Iterator
+from typing import NamedTuple
 
-from gaphor.core.modeling.element import Id
+from gaphor.core.modeling import Id
 from gaphor.diagram.copypaste import (
-    ElementCopy,
+    BaseCopy,
+    Opaque,
     copy,
+    copy_diagram,
     copy_element,
     paste,
     paste_element,
 )
 from gaphor.UML.recipes import owner_package
-from gaphor.UML.uml import NamedElement, Relationship
+from gaphor.UML.uml import (
+    Diagram,
+    MultiplicityElement,
+    NamedElement,
+    Package,
+    Relationship,
+    Slot,
+    Type,
+)
+
+
+def copy_multiplicity(
+    element: MultiplicityElement, blacklist: list[str] | None = None
+) -> Iterator[tuple[Id, Opaque]]:
+    if element.lowerValue:
+        yield from copy(element.lowerValue)
+    if element.upperValue:
+        yield from copy(element.upperValue)
 
 
 class NamedElementCopy(NamedTuple):
-    element_copy: ElementCopy
+    element_copy: BaseCopy
 
 
 def copy_named_element(
@@ -31,6 +51,11 @@ def _copy_named_element(element: NamedElement) -> Iterator[tuple[Id, NamedElemen
     yield element.id, copy_named_element(element)
 
 
+@copy.register
+def _copy_diagram(element: Diagram) -> Iterator[tuple[Id, Opaque]]:
+    yield from copy_diagram(element)
+
+
 def paste_named_element(copy_data: NamedElementCopy, diagram, lookup):
     paster = paste_element(
         copy_data.element_copy,
@@ -42,11 +67,21 @@ def paste_named_element(copy_data: NamedElementCopy, diagram, lookup):
     yield element
     next(paster, None)
     if (
-        not element.namespace
-        and hasattr(element, "package")
+        isinstance(element, Type | Package)
+        and (not element.namespace)
         and (package := owner_package(diagram.owner)) is not element
     ):
-        element.package = package
+        if isinstance(element, Package):
+            element.nestingPackage = package
+        else:
+            element.package = package
 
 
 paste.register(NamedElementCopy, paste_named_element)
+
+
+@copy.register
+def copy_slot(element: Slot):
+    yield element.id, copy_element(element)
+    if element.value:
+        yield from copy(element.value)
