@@ -24,8 +24,8 @@ class InteractionItem(Named, ElementPresentation):
             draw=draw_interaction,
         )
 
-        self.watch("subject.name")
-        self.watch("subject.appliedStereotype.classifier.name")
+        self.watch("subject[NamedElement].name")
+        self.watch("subject[Element].appliedStereotype.classifier.name")
 
 
 def draw_interaction(box, context, bounding_box):
